#include "global.h"

char bit_matrix_column_double(bit_matrix *r)
{
  int i, j;
  char *tmp;
  
  r->max_columns *= 2;
  for (i=0; i<r->number_of_rows; i++)
    {
      tmp = (char *)calloc(r->max_columns, sizeof(char));
      if (!tmp)
	return 0;
      
      for (j=0; j<r->number_of_columns; j++)
	tmp[j] = r->matrix[i][j];

      free(r->matrix[i]);
      r->matrix[i] = tmp;
    }
  
  return 1;
}

static char bit_matrix_row_double(bit_matrix *r)
{
  int i;
  char **tmp;

  r->max_rows *= 2;
  tmp = (char **)malloc(r->max_rows*sizeof(char *));
  if (!tmp)
    return 0;

  for (i=0; i<r->number_of_rows; i++)
    tmp[i] = r->matrix[i];

  free(r->matrix);
  r->matrix = tmp;
  
  return 1;
}

bit_matrix *bit_matrix_init()
{
  bit_matrix *r;
  
  r = (bit_matrix *)malloc(sizeof(bit_matrix));
  if (!r)
    return NULL;
  
  r->number_of_rows = 0;
  r->number_of_columns = 0;
  r->max_rows = MEMORY_CHUNK;
  r->max_columns = MEMORY_CHUNK;
  
  r->matrix = (char **)malloc(r->max_rows*sizeof(char *));
  if (!r->matrix)
    {
      free(r);
      return NULL;
    }
  
  return r;
}

void bit_matrix_free(bit_matrix *r)
{
  int i;
  
  for (i=0; i<r->number_of_rows; i++)
    free(r->matrix[i]);
  free(r->matrix);
  free(r);
}

char bit_matrix_add_row(bit_matrix *b)
{
  if (b->number_of_rows == b->max_rows)
    {
      if (!bit_matrix_row_double(b))
	return 0;
    }
  
  b->matrix[b->number_of_rows] = calloc(b->max_columns, sizeof(char));
  if (!b)
    return 0;
  
  b->number_of_rows++;
  return 1;
}

void bit_matrix_print(bit_matrix *r)
{
  int i, j;
  
  printf("Printing Bit Matrix:\n");

  for (i=0; i<r->number_of_rows; i++)
    {
      for (j=0; j<r->number_of_columns; j++)
	printf("%6d  ", r->matrix[i][j]);
      printf("\n");
    }
}

int *bit_matrix_sum_columns(bit_matrix *b, char *bit_array)
{
  int *sum;
  int i, j;
  
  sum = (int *)calloc(b->number_of_rows,sizeof(int));
  if (!sum)
    return NULL;

  for (j=0; j<b->number_of_columns; j++)
    if (bit_array[j])
      {
	for (i=0; i<b->number_of_rows; i++)
	  sum[i] += b->matrix[i][j];
      }
  
  return sum;
}
