#include "global.h"
#include "init.h"
#include "relaxation.h"
#include "subset.h"
#include "set_covering.h"

int num_demand_points;
int num_service_set_size;
int num_service_points;
int give_up = 50;
int restart_size = 4;
int seed = 345345;

double radius_upper_limit = DBL_MAX;

static void print_current_best_solution(relaxation_matrix *r, subset *s,
				 double solution_value)
{
#ifdef VERBOSE
  int i;
#endif

  printf("Current best solution: %lf\n", solution_value);
#ifdef VERBOSE
  for (i=0; i<s->size; i++)
    {
      printf("index: %d\n", r->circles[s->indices[i]].index);
      printf("r: %6lf\n\n", r->circles[s->indices[i]].r);
    }
#endif
  fflush(stdout);
}

int main(int argc, char *argv[])
{
  subset reduced, solution_reduced;
  relaxation_matrix r;
  int farthest;
  double solution_value;
  int k;
  int start_over, start_service_points;

  if((argc < 3) || (argc > 8))
    {
      fprintf(stderr, "usage: %s filename "
	      "num_service_points [guess] "
	      "[restart_size] [give_up] [start_service_points] [seed]\n",
	      argv[0]);
      exit(0);
    }
  
  num_service_points = start_service_points = atoi(argv[2]);

  if (argc >= 4)
      sscanf(argv[3], "%lf", &radius_upper_limit);
  else
    radius_upper_limit = DBL_MAX;
  
  if (argc >= 5)
    restart_size = atoi(argv[4]);

  if (argc >= 6)
    give_up = atoi(argv[5]);

  if (argc >= 7)
    start_service_points = atoi(argv[6]);

  if (argc >= 8)
    seed = atoi(argv[7]);

  srandom(seed);
  
  init_point_array(argv[1]);
  
  subset_init(&solution_reduced);
  
  subset_init(&reduced);
  subset_random(&reduced, 1);
  
  relaxation_matrix_init(&r, &reduced);
  
  for (k=start_service_points; k<=num_service_points ; k++)
    {
      start_over = 0;
    Start_over:
      if (start_over)
	{
	  subset_free(&reduced);
	  subset_init(&reduced);
	  relaxation_matrix_free(&r);
	  subset_random(&reduced, restart_size);
	  relaxation_matrix_init(&r, &reduced);
	}
      else
	start_over = 1;
      
      while (1)
	{
	  if (set_covering(&r, &solution_reduced) == -1)
	    goto Start_over;

	  if (solution_reduced.size > k)
	    goto Cont;

	  solution_value = get_solution_value(&r, &solution_reduced);
	  
	  if (check_feasibility(&r, &solution_reduced, &farthest,
				solution_value))
	    {
	      radius_upper_limit = solution_value / (1 + EPSILON);
	      print_current_best_solution(&r, &solution_reduced,
					  solution_value);
	      goto Start_over;
	    }
	  else
	    {
	      subset_add(&reduced, farthest);
	      relaxation_matrix_free(&r);
	      relaxation_matrix_init(&r, &reduced);
	    }
	}
    Cont:
      printf("Okay, we have solved the problem for %d service points\n", k);
      printf("------------------------------------------------------\n");
      fflush(stdout);
    }

  return 1;
}
