#include <string.h>
#include "global.h"
#include "util.h"
#include "bit_matrix.h"

static char relaxation_matrix_double(relaxation_matrix *r)
{
  int j;
  circle *tmp1;
  
  r->total_size *= 2;
  
  r->bit->number_of_columns = r->number_of_circles;
  if (!bit_matrix_column_double(r->bit))
    return 0;
  
  tmp1 = (circle *)malloc(r->total_size*sizeof(circle));
  if (!tmp1)
    return 0;
  
  for (j=0; j<r->number_of_circles; j++)
    circle_copy(&tmp1[j], &r->circles[j]);

  free (r->circles);
  r->circles = tmp1;
  
  return 1;
}

char relaxation_matrix_init(relaxation_matrix *r, subset *s)
{
  int i,j,z;
  circle c;
  double dummy;

  r->number_of_points = s->size;
  r->number_of_circles = 0;
  r->bit = bit_matrix_init();
  if (!r->bit)
    return 0;
  r->total_size = MEMORY_CHUNK;

  r->circles = (circle *)malloc(MEMORY_CHUNK*sizeof(circle));
  
  for (i=0; i<s->size; i++)
    bit_matrix_add_row(r->bit);

  for (j=0; j<num_service_set_size; j++)    
    {
      double tmp = 0;
      
      for (i=0; i<s->size; i++)
	{
	  simple_circle(s->indices[i], j, &c);
	  if (c.r > radius_upper_limit)
	    continue;
	  tmp = c.r > tmp ? c.r : tmp;
	}
      
      c.r = tmp;
      c.index = j;
      
      if (r->number_of_circles == r->total_size)
	{
	  if (!relaxation_matrix_double(r))
	    return 0;
	}
      
      for (z=0; z<r->number_of_points; z++)
	r->bit->matrix[z][r->number_of_circles] =
	  in_circle(&c, s->indices[z], &dummy);
      circle_copy(&r->circles[r->number_of_circles], &c);
      r->number_of_circles++;
    }

  r->bit->number_of_columns = r->number_of_circles;

  return 1;
}

void relaxation_matrix_print(relaxation_matrix *r)
{
  int i, j;
  
  printf("Printing Relaxation Matrix:\n");

  for (i=0; i<r->number_of_points; i++)
    {
      for (j=0; j<r->number_of_circles; j++)
	printf("%6d  ", r->bit->matrix[i][j]);
      printf("\n");
    }
  for (j=0; j<r->number_of_circles; j++)
    printf("%6.3lf  ", r->circles[j].r);
  printf("\n");
  
  for (j=0; j<r->number_of_circles; j++)
    printf("%d  ", r->circles[j].index);
  printf("\n");
}

char check_feasibility(relaxation_matrix *rel, subset *s, int *farthest,
		       double solution_value)
{
  int i, j;
  double max_min_dist = 0, r;
  
  for (i=0; i<num_demand_points; i++)
    {
      double min_dist = DBL_MAX;
      
      for (j=0; j<s->size; j++)
	{
	  in_circle(&rel->circles[s->indices[j]], i, &r);
	  if (r < min_dist)
	    min_dist = r;
	  if (r <= solution_value)
	    break;
	}
      
      if (min_dist > max_min_dist)
	{
	  *farthest = i;
	  max_min_dist = min_dist;
	}
    }
  
  if (max_min_dist == 0)
    return 1;

  if (max_min_dist <= solution_value * (1 + EPSILON))
    return 1;
  else
    return 0;
}

void relaxation_matrix_free(relaxation_matrix *r)
{
  free(r->circles);
  bit_matrix_free(r->bit);
}

double get_solution_value(relaxation_matrix *r, subset *s)
{
  double ret = 0, tmp;
  int i;

  for (i=0; i<s->size; i++)
    {
      tmp = r->circles[s->indices[i]].r;
      if (tmp > ret)
	ret = tmp;
    }

  return ret;
}

